#!/bin/bash
VERSION='2023_1'
INSTALLER_VERSION='2.0'
FILE='install.zip'
DRONE_ENGAGE_FOLDER_NAME='drone_engage'
DRONE_COMMUNICATIOR_FOLDER_NAME='de_comm'
DRONE_MAVLINK_FOLDER_NAME='de_mavlink'
DRONE_COMMUNICATIOR_BIN_NAME='de_comm.so'
DRONE_MAVLINK_BIN_NAME='de_ardupilot.so'
DRONE_COMMUNICATIOR_CONFIG_NAME='de_comm.config.module.json'
DRONE_MAVLINK_CONFIG_NAME='de_mavlink.config.module.json'

RED='\033[1;31m'
GREEN='\033[1;32m'
YELLOW='\033[1;33m'
BLUE='\033[1;34m'
NC='\033[0m' # No Color
TMP_OUTPUT=$(mktemp -d -u)
echo -e $GREEN "* DroneEngage Intaller version $INSTALLER_VERSION."  $NC



if [ "$(whoami)" != "pi" ]; then
        echo "Script must be run as user: pi"
        sudo rm -rf $TMP_OUTPUT
        exit 255
fi

if ! command -v jq &> /dev/null
then
    sudo apt install  jq
    sudo rm -rf $TMP_OUTPUT
    echo "Please rerun the script"
    exit
fi


##############################################################################
# Unzip Install Files
echo -e $GREEN "* Unzip file $FILE in $TMP_OUTPUT" $NC
echo -e $GREEN "--------------------------------------------" $NC
unzip -qq -o  $FILE  -d $TMP_OUTPUT 

pushd $TMP_OUTPUT


##############################################################################
# Binary Installation

echo -e $GREEN "* Copying Binaries & Checking Binaries Version" $NC

DESTINATION_FOLDER="/home/pi/$DRONE_ENGAGE_FOLDER_NAME"
mkdir -p $DESTINATION_FOLDER

### Communicator Binary

####### create folder
mkdir -p $DESTINATION_FOLDER/de_comm

####### backup current file
if [ -f "$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME" ]; then
    echo -e $YELLOW "Backup current binaries... $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME.bak"  $NC
    mv $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME       $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME.bak
fi
####### copy new file
chmod +x ./drone_engage/de_comm/de_comm.so
cp ./drone_engage/de_comm/de_comm.so $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME
####### run binary --version and display the version
echo -e $BLUE "-----running $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME -v" $NC
$DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME -v

######
######
######
######

### Mavlink Binary
####### create folder
mkdir -p $DESTINATION_FOLDER/de_mavlink
####### backup current file
if [ -f "$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME" ]; then
    echo -e $YELLOW "Backup current binaries... $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME.bak"  $NC
    mv $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME       $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME.bak
fi
####### copy new file
chmod +x ./drone_engage/de_mavlink/de_ardupilot.so
cp ./drone_engage/de_mavlink/de_ardupilot.so $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME
####### run binary --version and display the version
echo -e $BLUE "-----running $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME -v" $NC
$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME -v

##
##############################################################################

##############################################################################
# Config Files Installation
echo ""
echo ""
echo -e $GREEN "* Copying Configuration Files" $NC

## Reads a json param from a json file
read_param () {
	local file_name=$1
    param_name=$2
	
	
    VALUE=$(cat $file_name | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq .$param_name)
    ## Normal print
    echo -e  $YELLOW"read " $GREEN"$param_name"$YELLOW":"$GREEN"$VALUE" $NC >&2
    ## Return 
    echo $VALUE
}


## Writes a json param to json file
write_param () {
	local file_name=$1
    param_name=$2
	param_value=$3
	
    RET=$(cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=$param_value" > $file_name.$param_name)
    cat $file_name.$param_name > $file_name
    rm $file_name.$param_name
    #echo -e "$RET"
    #$RET
}

write_param2 () {
	local file_name=$1
    param_name=$2
	param_value=$3
	#echo "cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=\"$param_value\"" > $file_name.$param_name"
    RET=$(cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=\"$param_value\"" > $file_name.$param_name)
    cat $file_name.$param_name > $file_name
    rm $file_name.$param_name
}

## ASK for parameter
ask_param () {
    title=$1
    question=$2
    default_value=$3

    RES=$(whiptail --inputbox "$question" 8 39 $default_value --title "$title" 3>&1 1>&2 2>&3)
                                                                        # A trick to swap stdout and stderr.
    # Again, you can pack this inside if, but it seems really long for some 80-col terminal users.
    exitstatus=$?
    if [ $exitstatus = 0 ]; then
        echo $RES
        return 
    else
        echo $default_value
        return 
    fi

    echo $default_value
}

copy_and_ask_param()
{
    local file_name=$1
    param_name=$2
    title=$3
    is_num=$4

    VALUE=$(read_param "$file_name" "$param_name")
    VALUE="${VALUE//\"}"  #remove ""
    VALUE=$(ask_param "$param_name" "$title" $VALUE)
    if [[ $is_num -eq 1 ]]; then
    write_param "$file_name" "$param_name" "$VALUE"
    else
    write_param2 "$file_name" "$param_name" "$VALUE"
    fi
}


#### Backup Comm Config File
FULL_COMM_CONFIG_PATH=$(echo "$DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_CONFIG_NAME")
if [ -f $FULL_COMM_CONFIG_PATH ]; then
    echo -e $YELLOW "Backup current config... $FULL_COMM_CONFIG_PATH.bak"  $NC
    mv $FULL_COMM_CONFIG_PATH       $FULL_COMM_CONFIG_PATH.bak
else
    echo -e $GREEN "Creating configuration file at:"  $YELLOW  $FULL_COMM_CONFIG_PATH $NC

fi
cp ./drone_engage/de_comm/$DRONE_COMMUNICATIOR_CONFIG_NAME $FULL_COMM_CONFIG_PATH
cp ./drone_engage/de_comm/root.crt $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/root.crt

if [ -f $FULL_COMM_CONFIG_PATH.bak ]; then
echo -e $GREEN "Previous settings are backed in $FULL_COMM_CONFIG_PATH.bak" $NC
#Copy Parameters
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "module_id")
write_param "$FULL_COMM_CONFIG_PATH" "module_id" "$VALUE"

VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "s2s_udp_listening_ip")
write_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_ip" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "s2s_udp_listening_port")
write_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_port" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "auth_ip")
write_param "$FULL_COMM_CONFIG_PATH" "auth_ip" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "auth_port")
write_param "$FULL_COMM_CONFIG_PATH" "auth_port" $VALUE
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "auth_verify_ssl")
write_param "$FULL_COMM_CONFIG_PATH" "auth_verify_ssl" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "root_certificate_path")
write_param "$FULL_COMM_CONFIG_PATH" "root_certificate_path" "$VALUE"

VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "userName")
write_param "$FULL_COMM_CONFIG_PATH" "userName" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "accessCode")
write_param "$FULL_COMM_CONFIG_PATH" "accessCode" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "unitID")
write_param "$FULL_COMM_CONFIG_PATH" "unitID" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "unitDescription")
write_param "$FULL_COMM_CONFIG_PATH" "unitDescription" "$VALUE"

VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "logger_enabled")
write_param "$FULL_COMM_CONFIG_PATH" "logger_enabled" "$VALUE"
VALUE=$(read_param "$FULL_COMM_CONFIG_PATH.bak" "logger_debug")
write_param "$FULL_COMM_CONFIG_PATH" "logger_debug" "$VALUE"
fi


echo -e $YELLOW
read -p "Do you want to edit parameter values (y/n)? " -n 1 -r
echo -e $NC    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "module_id" "Communication Module Name" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_ip" "Communication Module UDP IP" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_port" "Communication Module UDP Port" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "auth_ip" "Server IP" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "auth_port" "Server Port" 1
fi

echo ""
echo "for full edit use:"
echo "nano $FULL_COMM_CONFIG_PATH"
echo ""
echo ""
read -p "press ENTER to continue"
########################################################################
########################################################################

#### Backup mavlink Config File
echo ""
echo ""
FULL_MAVLINK_CONFIG_PATH=$(echo "$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_CONFIG_NAME")
if [ -f $FULL_MAVLINK_CONFIG_PATH ]; then
    echo -e $YELLOW "Backup current mavlink config... $FULL_MAVLINK_CONFIG_PATH.bak"  $NC
    mv $FULL_MAVLINK_CONFIG_PATH     $FULL_MAVLINK_CONFIG_PATH.bak
else
    echo -e $GREEN "Creating configuration file at:"  $YELLOW  $FULL_MAVLINK_CONFIG_PATH $NC
fi
cp ./drone_engage/de_mavlink/$DRONE_MAVLINK_CONFIG_NAME $FULL_MAVLINK_CONFIG_PATH

if [ -f $FULL_MAVLINK_CONFIG_PATH.bak ]; then
echo -e $GREEN "Previous settings are backed in $FULL_MAVLINK_CONFIG_PATH.bak" $NC
echo -e $YELLOW
#Copy Parameters
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "module_id")
write_param "$FULL_MAVLINK_CONFIG_PATH" "module_id" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "s2s_udp_target_ip")
write_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_ip" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "s2s_udp_target_port")
write_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_port" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "s2s_udp_listening_ip")
write_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_ip" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "s2s_udp_listening_port")
write_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_port" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "fcb_connection_uri")
write_param "$FULL_MAVLINK_CONFIG_PATH" "fcb_connection_uri" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "logger_enabled")
if [ VALUE==null ]; then
VALUE=false
fi
write_param "$FULL_MAVLINK_CONFIG_PATH" "logger_enabled" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "logger_debug")
if [ VALUE==null ]; then
VALUE=false
fi
write_param "$FULL_MAVLINK_CONFIG_PATH" "logger_debug" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "event_fire_channel")
write_param "$FULL_MAVLINK_CONFIG_PATH" "event_fire_channel" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "event_wait_channel")
write_param "$FULL_MAVLINK_CONFIG_PATH" "event_wait_channel" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "read_only_mode")
write_param "$FULL_MAVLINK_CONFIG_PATH" "read_only_mode" "$VALUE"
VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "ignore_loading_parameters")
write_param "$FULL_MAVLINK_CONFIG_PATH" "ignore_loading_parameters" "$VALUE"
fi

echo -e $YELLOW
read -p "Do you want to edit parameter values (y/n)? " 
echo -e $NC    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "module_id" "Mavlink Module Name" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_ip" "Communication Module UDP IP" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_port" "Communication Module UDP Port" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_ip" "Mavlink Module UDP IP" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_port" "Mavlink Module UDP Port" 0
fi


echo ""
echo "for full edit use:"
echo "nano $FULL_MAVLINK_CONFIG_PATH"
echo ""
echo ""
read -p "press ENTER to continue"

##
##############################################################################


##############################################################################
# Service Installation
service_exists() {
    local n=$1
    if [[ $(systemctl list-units --all -t service --full --no-legend "$n.service" | sed 's/^\s*//g' | cut -f1 -d' ') == $n.service ]]; then
        return 0
    else
        return 1
    fi
}

echo -e $GREEN "--------------------------------------------" $NC

if service_exists de_communicator; then
    echo -e $YELLOW "stopping service de_communicator.service" $NC
    sudo systemctl stop de_communicator.service 
fi

if service_exists de_mavlink; then
    echo -e $YELLOW "stopping service de_mavlink.service" $NC
    sudo systemctl stop de_mavlink.service 
fi


sudo mv ./de_communicator.service  /lib/systemd/system/de_communicator.service
sudo mv ./de_mavlink.service  /lib/systemd/system/de_mavlink.service

sudo chmod +x ./drone_engage/de_comm/de_comm.so
sudo chmod +x ./drone_engage/de_mavlink/de_ardupilot.so

echo -e $GREEN "Enabling de_communicator.service" $NC
sudo systemctl enable de_communicator.service 
echo -e $GREEN "Enabling de_mavlink.service" $NC
sudo systemctl enable de_mavlink.service
sudo systemctl daemon-reload 
sudo systemctl restart de_communicator.service 
sudo systemctl restart de_mavlink.service



if [ $(systemctl is-active de_communicator.service) == "active" ] ; then
    echo -e $GREEN "service de_communicator.service is active" $NC
else
    echo -e $RED "WARNING: " $YELLOW "service de_communicator.service is not active" $NC
fi

if [ $(systemctl is-active de_mavlink.service) == "active" ] ; then
    echo -e $GREEN "service de_mavlink.service is active" $NC
else
    echo -e $RED "WARNING: " $YELLOW "service de_mavlink.service is not active" $NC
fi
##
##############################################################################

sudo rm -rf $TMP_OUTPUT

echo "DONE"        

